@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-encoding.c,v 1.75 2007/04/14 19:36:35 pkot Exp $
+  $Id: gsm-encoding.c,v 1.83 2008/04/05 15:40:01 dforsi Exp $
 
   G N O K I I
 
@@ -32,11 +32,12 @@
 
 */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 #include "compat.h"
 #include "misc.h"
 #include "gnokii.h"
@@ -57,6 +58,8 @@ extern const char *locale_charset(void); /* from ../intl/localcharset.c */
 #endif
 
 static const char *base64_alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+/* BCD digits from Table 10.5.118 of 3GPP TS 04.08 with 'a' replaced by 'p' */
+static const char *bcd_digits = "0123456789*#pbc";
 
 #define GN_CHAR_ALPHABET_SIZE 128
 
@@ -119,7 +122,7 @@ static const char *get_langinfo_codeset(void)
 #  ifdef WIN32
 		/* As suggested by Ben Bryant, http://codesnipers.com/?q=node/46 */
 		char szCP[10];
-		sprintf(szCP, ".%d", GetACP());
+		snprintf(szCP, sizeof(szCP), ".%d", GetACP());
 		codeset = setlocale(LC_ALL, szCP);
 #  else
 		codeset = locale_charset();
@@ -149,7 +152,7 @@ static int char_mbtowc(wchar_t *dst, const char *src, int maxlen, MBSTATE *mbs)
 {
 #ifdef HAVE_ICONV
 	size_t nconv;
-	char *pin;
+	ICONV_CONST char *pin;
 	char *pout;
 	size_t inlen;
 	size_t outlen;
@@ -178,7 +181,7 @@ static int char_wctomb(char *dst, wchar_t src, MBSTATE *mbs)
 {
 #ifdef HAVE_ICONV
 	size_t nconv;
-	char *pin;
+	ICONV_CONST char *pin;
 	char *pout;
 	size_t inlen;
 	size_t outlen;
@@ -435,7 +438,7 @@ size_t char_hex_encode(char *dest, size_t dest_len, const char *src, size_t len)
 	int i, n = dest_len / 2 >= len ? len : dest_len / 2;
 
 	for (i = 0; i < n; i++)
-		sprintf(dest + i * 2, "%x", char_def_alphabet_encode(src[i]));
+		snprintf(dest + i * 2, 3, "%02x", char_def_alphabet_encode(src[i]));
 	return len * 2;
 }
 
@@ -508,7 +511,7 @@ size_t char_ucs2_encode(char *dest, size_t dest_len, const char *src, size_t len
 			return i * 4;
 		i += length;
 		/* XXX: We should probably check wchar_t size. */
-		sprintf(dest + (o_len << 2), "%04x", wc);
+		snprintf(dest + (o_len << 2), 5, "%04x", wc);
 	}
 	return len * 4;
 }
@@ -649,30 +652,32 @@ char *char_bcd_number_get(u8 *number)
 	int count, digit, i = 0;
 
 	if (length > GN_BCD_STRING_MAX_LENGTH) length = GN_BCD_STRING_MAX_LENGTH;
-	memset(buffer, 0, GN_BCD_STRING_MAX_LENGTH);
 	switch (number[1]) {
 	case GN_GSM_NUMBER_Alphanumeric:
 		char_7bit_unpack(0, length, length, number + 2, buffer);
 		buffer[length] = 0;
 		break;
 	case GN_GSM_NUMBER_International:
-		sprintf(buffer, "+");
+		snprintf(buffer, sizeof(buffer), "+");
 		i++;
-		if (length == GN_BCD_STRING_MAX_LENGTH) length--; /* avoid overflow */
+		if (length == GN_BCD_STRING_MAX_LENGTH)
+			length--; /* avoid overflow */
 	case GN_GSM_NUMBER_Unknown:
 	case GN_GSM_NUMBER_National:
 	case GN_GSM_NUMBER_Network:
 	case GN_GSM_NUMBER_Subscriber:
 	case GN_GSM_NUMBER_Abbreviated:
 	default:
-		for (count = 0; count < length - 1; count++) {
-			digit = number[count+2] & 0x0f;
-			if (digit < 10)
-				buffer[i++] = digit + '0';
-			digit = number[count+2] >> 4;
-			if (digit < 10)
-				buffer[i++] = digit + '0';
+		/* start at 2 to skip length and TON (we can't overflow the buffer because i <= GN_BCD_STRING_MAX_LENGTH - 2) */
+		for (count = 2; count <= length; count++) {
+			digit = number[count] & 0x0f;
+			if (digit < 0x0f)
+				buffer[i++] = bcd_digits[digit];
+			digit = number[count] >> 4;
+			if (digit < 0x0f)
+				buffer[i++] = bcd_digits[digit];
 		}
+		buffer[i] = '\0';
 		break;
 	}
 	return buffer;
@@ -680,8 +685,7 @@ char *char_bcd_number_get(u8 *number)
 
 void gn_char_set_encoding(const char* encoding)
 {
-	strncpy(application_encoding, encoding, sizeof(application_encoding) - 1);
-	application_encoding[sizeof(application_encoding) - 1] = 0;
+	snprintf(application_encoding, sizeof(application_encoding), "%s", encoding);
 }
 
 /* UTF-8 conversion functions */
@@ -792,43 +796,41 @@ int string_base64(const char *instring)
 }
 
 /*
-   encodes a null-terminated input string with base64 encoding.
-   the buffer outstring needs to be at least 1.333 times bigger than the input string length
+   Encodes a null-terminated input string with base64 encoding.
+   The buffer outstring needs to be at least 1.333 times bigger than the input string length.
+   outlen is the size of the outstring buffer excluding null termination.
 */
 int base64_encode(char *outstring, int outlen, const char *instring, int inlen)
 {
-	char *in1, *pin, *pout;
+	const char *pin;
+	char *pout;
 	char *outtemp = NULL;
-	int inleft, outleft, inprocessed;
-	unsigned int i1, i2, i3, i4;
-
-
-	/* copy the input string, need multiple of 3 chars */
-	in1 = malloc(inlen + 3);
-	memset(in1, 0, inlen + 3);
-	memcpy(in1, instring, inlen);
+	int inleft, outleft;
 
 	pout = outstring;
 	inleft = inlen;
 	outleft = outlen;
-	inprocessed = 0;
-	pin = in1;
+	pin = instring;
 
-	for (; (outleft > 3) && (inprocessed < inlen); ) {
-		if (!(*pin))
-			break;
+	/* This is in case someone passes a buffer not appropriate for outstring */
+	while (outleft > 3 && inleft > 0) {
+		int a, b, c;
+		unsigned int i1, i2, i3, i4;
+
+		a = *pin++;
+		b = (*pin) ? *(pin++) : 0;
+		c = (*pin) ? *(pin++) : 0;
 
 		/* calculate the indexes */
-		i1 = (pin[0] & 0xFC) >> 2;
-		i2 = ((pin[0] & 0x03) << 4) | ((pin[1] & 0xF0) >> 4);
-		i3 = ((pin[1] & 0x0F) << 2) | ((pin[2] & 0xC0) >> 6);
-		i4 = pin[2] & 0x3F;
-		pin += 3;
-
-		/* assign the characters or the padding char, if necessary */
+		i1 = (a & 0xfc) >> 2;
 		*(pout++) = base64_alphabet[i1];
+
+		i2 = ((a & 0x03) << 4) | ((b & 0xf0) >> 4);
 		*(pout++) = base64_alphabet[i2];
+
 		inleft--;
+
+		i3 = ((b & 0x0f) << 2) | ((c & 0xc0) >> 6);
 		if (!inleft) {
 			*(pout++) = '=';
 		} else {
@@ -836,6 +838,7 @@ int base64_encode(char *outstring, int outlen, const char *instring, int inlen)
 			inleft--;
 		}
 
+		i4 = c & 0x3f;
 		if (!inleft)
 			*(pout++) = '=';
 		else {
@@ -843,17 +846,14 @@ int base64_encode(char *outstring, int outlen, const char *instring, int inlen)
 			inleft--;
 		}
 
-		/* update the counters */
-		inprocessed += 3;
 		outleft -= 4;
 	}
 
-	if (outleft > 0) *pout = 0;
+	*pout = 0;
+
 	if (outtemp)
 		free(outtemp);
 
-	free(in1);
-
 	return pout - outstring;
 }
 
@@ -989,9 +989,9 @@ int strip_slashes(char *dest, char *src, int maxlen, int len)
 		case ';':
 		case ',':
 			if (slash_state) {
-				dest[j++] = src[i];
 				slash_state = 0;
 			}
+			dest[j++] = src[i];
 			break;
 		case '\\':
 			if (slash_state) {
@@ -1021,9 +1021,8 @@ int strip_slashes(char *dest, char *src, int maxlen, int len)
 			if (slash_state) {
 				dest[j++] = '\\';
 				slash_state = 0;
-			} else {
-				dest[j++] = src[i];
 			}
+			dest[j++] = src[i];
 			break;
 		}
 	}
