@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii.c,v 1.480 2007/05/26 13:48:54 dforsi Exp $
+  $Id: gnokii.c,v 1.483 2008/05/08 13:37:05 dforsi Exp $
 
   G N O K I I
 
@@ -379,8 +379,9 @@ static int foogle(int argc, char *argv[])
 
 static int parse_options(int argc, char *argv[])
 {
-	int c, rc = -1;
+	int c;
 	int opt_index = -1;
+	gn_error rc;
 
 	/* Every option should be in this array. */
 	static struct option long_options[] = {
@@ -640,7 +641,7 @@ static int parse_options(int argc, char *argv[])
 		{ OPT_CONFIGFILE,        1, 100, 0 },
 		{ OPT_CONFIGMODEL,       1, 100, 0 },
 #ifdef SECURITY
-		{ OPT_ENTERSECURITYCODE, 1, 1, 0 },
+		{ OPT_ENTERSECURITYCODE, 1, 100, 0 },
 		{ OPT_CHANGESECURITYCODE,1, 1, 0 },
 #endif
 		{ OPT_SETDATETIME,       0, 5, 0 },
@@ -764,7 +765,7 @@ static int parse_options(int argc, char *argv[])
 	}
 
 	/* Initialise the code for the GSM interface. */
-	if (c != OPT_FOOGLE)
+	if (c != OPT_FOOGLE && state == NULL)
 		businit();
 
 	switch (c) {
@@ -965,6 +966,8 @@ static int parse_options(int argc, char *argv[])
 		break;
 	case OPT_ENTERSECURITYCODE:
 		rc = entersecuritycode(optarg, data, state);
+		if (rc == GN_ERR_NONE && optind < argc)
+			return parse_options(argc, argv);
 		break;
 	case OPT_GETSECURITYCODESTATUS:
 		rc = getsecuritycodestatus(data, state);
@@ -1022,7 +1025,7 @@ static int parse_options(int argc, char *argv[])
 		break;
 #endif
 	default:
-		rc = 1;
+		rc = GN_ERR_FAILED;
 		fprintf(stderr, _("Unknown option: %d\n"), c);
 		break;
 	}
@@ -1037,8 +1040,9 @@ int main(int argc, char *argv[])
 
 	/* For GNU gettext */
 #ifdef ENABLE_NLS
-	textdomain("gnokii");
 	setlocale(LC_ALL, "");
+	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
+	textdomain(GETTEXT_PACKAGE);
 #endif
 
 	opterr = 0;
