@@ -1,6 +1,6 @@
 /*
 
-  $Id: config.h,v 1.10 2007/07/24 20:29:19 pkot Exp $
+  $Id: config.h,v 1.20 2008/05/30 19:34:54 pkot Exp $
 
   G N O K I I
 
@@ -24,7 +24,7 @@
 
   Copyright (C) 2003 Marcus Godehardt
 
-  This is the config.h file for GNOKII for MSVC6 compilers, you can
+  This is the config.h file for GNOKII for MSVS2005 compilers, you can
   edit this manually. This is not auto generated.
 
 */
@@ -35,12 +35,11 @@
 #if defined(_MSC_VER) && defined(WIN32)
 #	pragma once
 #else
-#	error This config.h is only for MSVC6 compiler !!!
+#	error This config.h is only for MSVS2005 compiler !!!
 #endif
 
-
 #undef VERSION
-#define VERSION "0.6.18"
+#define VERSION "0.6.26"
 
 /* We support Bluetooth and IRDA on MSVS 2005 */
 #define HAVE_BLUETOOTH 1
